@@ -28,32 +28,34 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <sys/types.h>
-#include <sys/wait.h>
-#include <sys/time.h>
-#include <errno.h>
 #include <ctype.h>
 #include <pthread.h>
+#include <ws_utils.h>
 
 #define W1_DEVICES_PATH "/sys/bus/w1/devices"
 #define MAX_SENSORS 64
 #define MAX_PATH_LEN 512
 #define MAX_LINE_LEN 256
 
+/* Config file path */
+#define CONFIG_PATH "/etc/ws/sensors/w1therm.json"
+
 /* Boot config paths (Raspberry Pi) */
 #define BOOT_CONFIG_PATH "/boot/firmware/config.txt"
 #define BOOT_CONFIG_PATH_LEGACY "/boot/config.txt"
 
 /* w1-gpio overlay configuration */
-#define W1_OVERLAY_LINE "dtoverlay=w1-gpio;gpiopin=17;pullup=1"
+#define W1_OVERLAY_LINE "dtoverlay=w1-gpio,gpiopin=17,pullup=1"
+
+/* Version information - passed via -DVERSION from Makefile (extracted from debian/changelog) */
+#ifndef VERSION
+#define VERSION "unknown"
+#endif
 
 /* Error values in millidegrees */
-#define STARTUP_VALUE_RAW 85000
+#define STARTUP_VALUE_RAW 85000  /* 85.000°C startup/error value */
 #define INSUFFICIENT_POWER_RAW 127937  /* 127.937°C in millidegrees */
 
-/* Exit codes */
-#define EXIT_SUCCESS_CODE 0
-#define EXIT_IDENTIFY_CODE 60
-
 /* Sensor reading result */
 typedef struct {
     char sensor_id[64];
@@ -70,6 +72,14 @@ typedef struct {
     sensor_result_t *result;
 } thread_args_t;
 
+/* Sensor configuration from config file */
+typedef struct {
+    char *hw_id;        /* Hardware ID to match (e.g., "28-00000a1b2c3d") */
+    char *sensor_id;    /* Override sensor_id in output */
+    char *sensor_name;  /* Human-readable name */
+    int internal;       /* 1 if internal, 0 if external */
+} sensor_config_t;
+
 /* Supported w1_therm family codes */
 static const char *W1_THERM_FAMILY_CODES[] = {
     "10-",  /* W1_THERM_DS18S20 */
@@ -103,6 +113,68 @@ static const char *get_sensor_type(const char *sensor_id) {
     return "unknown";
 }
 
+/*
+ * Parse a simple JSON config file - returns dynamically allocated array
+ */
+static sensor_config_t *load_config(const char *path, int *count) {
+    *count = 0;
+    
+    char *buffer = ws_read_file(path, NULL);
+    if (!buffer) return NULL;
+    
+    int sensor_count = ws_json_count_objects(buffer);
+    if (sensor_count == 0) { free(buffer); return NULL; }
+    
+    sensor_config_t *configs = malloc(sensor_count * sizeof(sensor_config_t));
+    if (!configs) { free(buffer); return NULL; }
+    
+    char *ptr = buffer;
+    int sensor_idx = 0;
+    while ((ptr = strchr(ptr, '{')) != NULL && sensor_idx < sensor_count) {
+        char *end = strchr(ptr, '}');
+        if (!end) break;
+        
+        configs[sensor_idx].internal = ws_json_parse_bool(ptr, end, "internal", 0);
+        configs[sensor_idx].hw_id = ws_json_parse_string(ptr, end, "hw_id");
+        configs[sensor_idx].sensor_id = ws_json_parse_string(ptr, end, "sensor_id");
+        configs[sensor_idx].sensor_name = ws_json_parse_string(ptr, end, "sensor_name");
+        
+        sensor_idx++;
+        ptr = end + 1;
+    }
+    
+    free(buffer);
+    *count = sensor_idx;
+    return configs;
+}
+
+/*
+ * Free config array
+ */
+static void free_config(sensor_config_t *configs, int count) {
+    if (configs) {
+        for (int i = 0; i < count; i++) {
+            free(configs[i].hw_id);
+            free(configs[i].sensor_id);
+            free(configs[i].sensor_name);
+        }
+        free(configs);
+    }
+}
+
+/*
+ * Find config for a sensor by hardware ID
+ */
+static sensor_config_t *find_sensor_config(sensor_config_t *configs, int count, const char *hw_id) {
+    if (!configs || !hw_id) return NULL;
+    for (int i = 0; i < count; i++) {
+        if (configs[i].hw_id && strcmp(configs[i].hw_id, hw_id) == 0) {
+            return &configs[i];
+        }
+    }
+    return NULL;
+}
+
 /*
  * Check if a directory entry is a w1_bus_master folder
  */
@@ -160,58 +232,6 @@ static int trigger_bulk_read(const char master_path[MAX_PATH_LEN]) {
     return 0;
 }
 
-/*
- * Poll for bulk read completion
- * Returns: 0 = no bulk read pending, 1 = complete, -1 = still in progress
- * 
- * Note: Since trigger_bulk_read() blocks until conversion is complete,
- * this function is mainly useful for checking status when trigger
- * was initiated externally or for non-blocking scenarios.
- */
-static int poll_bulk_read(const char *master_path) {
-    char file_path[MAX_PATH_LEN];
-    FILE *fp;
-    int status;
-
-    snprintf(file_path, sizeof(file_path), "%s/therm_bulk_read", master_path);
-
-    fp = fopen(file_path, "r");
-    if (!fp) {
-        return 0;  /* No bulk read support */
-    }
-
-    if (fscanf(fp, "%d", &status) != 1) {
-        fclose(fp);
-        return 0;
-    }
-
-    fclose(fp);
-    return status;
-}
-
-/* Unused but kept for potential non-blocking scenarios */
-__attribute__((unused))
-static int wait_for_bulk_read(const char *master_path, int timeout_ms) {
-    int elapsed = 0;
-    int poll_interval_us = 10000;  /* 10ms poll interval */
-    int status;
-
-    while (elapsed < timeout_ms) {
-        status = poll_bulk_read(master_path);
-        if (status == 1) {
-            return 0;  /* Conversion complete */
-        }
-        if (status == 0) {
-            return 0;  /* No conversion pending */
-        }
-        /* status == -1: still in progress */
-        usleep(poll_interval_us);
-        elapsed += poll_interval_us / 1000;
-    }
-
-    return -1;  /* Timeout */
-}
-
 /*
  * Enable poll-for-conversion feature on a sensor
  * This makes the driver poll the bus for conversion completion
@@ -409,9 +429,8 @@ static void *read_sensor_thread(void *arg) {
         return NULL;
     }
 
-    /* Check for insufficient power value (~127.94°C) */
-    /* The raw value is approximately 127937 millidegrees */
-    if (temp_raw >= 127000 && temp_raw <= 128000) {
+    /* Check for insufficient power value (127.937°C) */
+    if (temp_raw == INSUFFICIENT_POWER_RAW) {
         result->has_error = 1;
         snprintf(result->error_msg, sizeof(result->error_msg), "Insufficient power");
         result->temperature = (double)temp_raw / 1000.0;
@@ -426,133 +445,40 @@ static void *read_sensor_thread(void *arg) {
 }
 
 /*
- * Escape a string for JSON output
+ * Print a single sensor result as JSON using library helper
  */
-static void json_escape_string(const char *src, char *dest, size_t dest_size) {
-    size_t i = 0, j = 0;
-    while (src[i] && j < dest_size - 2) {
-        if (src[i] == '"' || src[i] == '\\') {
-            dest[j++] = '\\';
-        }
-        dest[j++] = src[i++];
-    }
-    dest[j] = '\0';
-}
-
-/*
- * Call sc-prototype to get the JSON template
- * Returns 0 on success, -1 on failure
- */
-static int get_sc_prototype(char *buffer, size_t bufsize) {
-    FILE *fp;
-    char *newline;
-
-    fp = popen("sc-prototype", "r");
-    if (fp == NULL) {
-        return -1;
-    }
-
-    if (fgets(buffer, bufsize, fp) == NULL) {
-        pclose(fp);
-        return -1;
-    }
-
-    pclose(fp);
-
-    /* Remove trailing newline */
-    newline = strchr(buffer, '\n');
-    if (newline) {
-        *newline = '\0';
-    }
-
-    return 0;
-}
-
-/*
- * Replace a JSON null value with a string value
- * e.g. "sensor":null -> "sensor":"ds18b20"
- */
-static void json_replace_null_string(char *json, const char *key, const char *value) {
-    char search[128];
-    char replace[256];
-    char *pos;
-    size_t search_len, replace_len, tail_len;
-
-    snprintf(search, sizeof(search), "\"%s\":null", key);
-    snprintf(replace, sizeof(replace), "\"%s\":\"%s\"", key, value);
-
-    pos = strstr(json, search);
-    if (pos == NULL) {
-        return;
-    }
-
-    search_len = strlen(search);
-    replace_len = strlen(replace);
-    tail_len = strlen(pos + search_len);
-
-    /* Move tail to make room (or shrink) */
-    memmove(pos + replace_len, pos + search_len, tail_len + 1);
-    /* Copy replacement */
-    memcpy(pos, replace, replace_len);
-}
-
-/*
- * Replace a JSON null value with a number value
- * e.g. "value":null -> "value":23.456
- */
-static void json_replace_null_number(char *json, const char *key, double value) {
-    char search[128];
-    char replace[256];
-    char *pos;
-    size_t search_len, replace_len, tail_len;
-
-    snprintf(search, sizeof(search), "\"%s\":null", key);
-    snprintf(replace, sizeof(replace), "\"%s\":%.3f", key, value);
-
-    pos = strstr(json, search);
-    if (pos == NULL) {
-        return;
-    }
-
-    search_len = strlen(search);
-    replace_len = strlen(replace);
-    tail_len = strlen(pos + search_len);
-
-    memmove(pos + replace_len, pos + search_len, tail_len + 1);
-    memcpy(pos, replace, replace_len);
-}
-
-/*
- * Print a single sensor result as JSON using sc-prototype template
- */
-static void print_sensor_json(const sensor_result_t *result, int is_first, const char *json_template) {
+static void print_sensor_json(const sensor_result_t *result, int is_first, sensor_config_t *config) {
     char json[2048];
-    char escaped_id[128];
-    char escaped_error[256];
+    const char *sensor_id_to_use;
+    const char *sensor_name = NULL;
+    bool internal = false;
+    time_t now = time(NULL);
 
     if (!is_first) {
         printf(",");
     }
 
-    /* Start with the template */
-    strncpy(json, json_template, sizeof(json) - 1);
-    json[sizeof(json) - 1] = '\0';
-
-    /* Populate the fields */
-    json_escape_string(result->sensor_id, escaped_id, sizeof(escaped_id));
+    /* Determine sensor_id: use config override or default to hardware ID */
+    sensor_id_to_use = (config && config->sensor_id) ? config->sensor_id : result->sensor_id;
     
-    json_replace_null_string(json, "sensor", result->sensor_type);
-    json_replace_null_string(json, "measures", "temperature");
-    json_replace_null_string(json, "unit", "Celsius");
-    json_replace_null_string(json, "sensor_id", escaped_id);
-
-    if (result->has_error) {
-        json_escape_string(result->error_msg, escaped_error, sizeof(escaped_error));
-        json_replace_null_string(json, "error", escaped_error);
+    if (config) {
+        sensor_name = config->sensor_name;
+        internal = config->internal;
     }
     
-    /* Always output value if we have a temperature reading */
-    json_replace_null_number(json, "value", result->temperature);
+    /* Build base JSON with common fields */
+    if (ws_build_sensor_json_base(json, sizeof(json),
+                                   result->sensor_type, "temperature", "Celsius",
+                                   sensor_id_to_use, sensor_name,
+                                   internal, now) != 0) {
+        return;
+    }
+
+    /* Add value or error */
+    if (result->has_error) {
+        ws_sensor_json_set_error(json, result->error_msg);
+    }
+    ws_sensor_json_set_value(json, result->temperature, 3);
 
     printf("%s", json);
 }
@@ -641,10 +567,10 @@ static int enable_w1_interface(void) {
         return 1;
     }
 
-    fprintf(fp, "\n# 1-Wire interface for temperature sensors (added by sensor-w1therm)\n");
     if (!has_all_section) {
-        fprintf(fp, "[all]\n");
+        fprintf(fp, "\n[all]\n");
     }
+    fprintf(fp, "# 1-Wire interface for temperature sensors (added by sensor-w1therm)\n");
     fprintf(fp, "%s\n", W1_OVERLAY_LINE);
     fclose(fp);
 
@@ -689,27 +615,50 @@ int main(int argc, char *argv[]) {
     sensor_result_t results[MAX_SENSORS];
     thread_args_t thread_args[MAX_SENSORS];
     pthread_t threads[MAX_SENSORS];
-    char json_template[1024];
     int sensor_count;
     int master_count;
     int i;
     int output_count = 0;
     int is_first = 1;
+    ws_location_filter_t location_filter = WS_LOCATION_ALL;
+    sensor_config_t *configs = NULL;
+    int config_count = 0;
 
-    /* Handle 'identify' command */
-    if (argc > 1 && strcmp(argv[1], "identify") == 0) {
-        return EXIT_IDENTIFY_CODE;
-    }
-
-    /* Handle 'list' command */
-    if (argc > 1 && strcmp(argv[1], "list") == 0) {
-        printf("temperature\n");
-        return EXIT_SUCCESS_CODE;
-    }
-
-    /* Handle 'enable' command - add w1-gpio overlay to config.txt */
-    if (argc > 1 && strcmp(argv[1], "enable") == 0) {
-        return enable_w1_interface();
+    /* Handle command-line arguments */
+    if (argc > 1) {
+        if (strcmp(argv[1], "identify") == 0) {
+            ws_cmd_identify();
+        } else if (strcmp(argv[1], "list") == 0) {
+            ws_cmd_list_single("temperature");
+        } else if (strcmp(argv[1], "--version") == 0 || strcmp(argv[1], "-v") == 0 ||
+                   strcmp(argv[1], "version") == 0) {
+            ws_print_version("sensor-w1therm", VERSION);
+            return WS_EXIT_SUCCESS;
+        } else if (strcmp(argv[1], "enable") == 0) {
+            return enable_w1_interface();
+        } else if (strcmp(argv[1], "internal") == 0) {
+            location_filter = WS_LOCATION_INTERNAL;
+        } else if (strcmp(argv[1], "external") == 0) {
+            location_filter = WS_LOCATION_EXTERNAL;
+        } else if (strcmp(argv[1], "setup") == 0) {
+            /* Handled below after finding sensors */
+        } else if (strcmp(argv[1], "mock") == 0) {
+            /* Output mock data for testing without hardware */
+            char *serial = ws_get_serial_with_suffix("w1therm_mock");
+            time_t now = time(NULL);
+            char json[2048];
+            if (ws_build_sensor_json_base(json, sizeof(json), "ds18b20", "temperature", "Celsius",
+                                          serial, "Mock DS18B20", false, now) == 0) {
+                ws_sensor_json_set_value(json, 21.375, 3);
+                printf("[%s]\n", json);
+            }
+            free(serial);
+            return WS_EXIT_SUCCESS;
+        } else if (strcmp(argv[1], "all") != 0) {
+            fprintf(stderr, "Unknown command: %s\n", argv[1]);
+            fprintf(stderr, "Usage: sensor-w1therm [--version|identify|list|setup|enable|mock|internal|external|all]\n");
+            return WS_EXIT_INVALID_ARG;
+        }
     }
 
     /* Find all w1_therm sensors */
@@ -741,20 +690,17 @@ int main(int argc, char *argv[]) {
             }
         }
         printf("Setup complete. Sensors are now optimized for fast reading.\n");
-        return EXIT_SUCCESS_CODE;
+        return WS_EXIT_SUCCESS;
     }
 
     if (sensor_count == 0) {
         printf("[]\n");
         fprintf(stderr, "Warning: No w1_therm sensors detected. Please check your wiring and ensure 1-Wire is enabled.\n");
-        return EXIT_SUCCESS_CODE;
+        return WS_EXIT_SUCCESS;
     }
 
-    /* Get JSON template from sc-prototype */
-    if (get_sc_prototype(json_template, sizeof(json_template)) != 0) {
-        fprintf(stderr, "Error: Failed to get JSON template from sc-prototype\n");
-        return 1;
-    }
+    /* Load config file if it exists */
+    configs = load_config(CONFIG_PATH, &config_count);
 
     /* Initialize results */
     memset(results, 0, sizeof(results));
@@ -810,7 +756,18 @@ int main(int argc, char *argv[]) {
     printf("[");
     for (i = 0; i < sensor_count; i++) {
         if (results[i].valid) {
-            print_sensor_json(&results[i], is_first, json_template);
+            /* Find config for this sensor */
+            sensor_config_t *sensor_cfg = find_sensor_config(configs, config_count, results[i].sensor_id);
+            
+            /* Apply location filter */
+            if (location_filter == WS_LOCATION_INTERNAL && (!sensor_cfg || !sensor_cfg->internal)) {
+                continue;  /* Skip non-internal sensors */
+            }
+            if (location_filter == WS_LOCATION_EXTERNAL && sensor_cfg && sensor_cfg->internal) {
+                continue;  /* Skip internal sensors */
+            }
+            
+            print_sensor_json(&results[i], is_first, sensor_cfg);
             is_first = 0;
             output_count++;
         } else if (results[i].sensor_id[0] != '\0') {
@@ -819,9 +776,12 @@ int main(int argc, char *argv[]) {
     }
     printf("]\n");
 
+    /* Free config memory */
+    free_config(configs, config_count);
+
     if (output_count == 0) {
         fprintf(stderr, "Warning: No w1_therm sensors detected. Please check your wiring and ensure 1-Wire is enabled.\n");
     }
 
-    return EXIT_SUCCESS_CODE;
+    return WS_EXIT_SUCCESS;
 }
